##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) 2000, 2001, 2002 Control.com, Inc.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
""" Brain classes for ZSQL methods. """

from DocumentTemplate.DT_Util import html_quote
import StructuredText
from string import join, split, replace

class PostingBrain:
    """ Methods to imitate a Squishdot Posting. """

    def getSubject(self):
        """Trivial """
        return self.subject

    def thread_path(self):
        """ We don't need thread_path.  It was a Squishdot mistake. """
        return ''

    def date_posted(self):
        """Get a string date in the old Squishdot way."""
        return self.created.aCommon()

    def showSummary(self):
        """Note that the summary is returned as plain text, regardless
        of self.encoding."""
        return self.render(self.summary, 'Plain')

    def render(self, storedLines, format):
        if not storedLines:
            return ''
        if   format=='Plain':
            return join(map(html_quote, split(storedLines, '\n')),'<BR>\n')
        elif format=='HTML':
            return storedLines
        elif format=='STX':
            return str(StructuredText.html_with_references(storedLines, level=3))
        return 'Invalid Format!'

    def schema(self):
        """ For debugging """
        return self.__record_schema__

    # The following methods only work for a full posting.
    # Both full and partial posting use this brain class,
    # but full postings have additional attributes.

    def showBody(self):
        """Return the body formatted according to it's encoding."""
        return self.render(self.body, self.encoding)

    def desc_items(self):
        """Return latest list of replies. """
        # Hand this mess off to the PostingsTool!
        return self.postings_tool.descItems(self)

    # OK, the following will work for a partial posting,
    # but they're most likely to be called with a full one.

    # These methods would like to add attributes to the class instance,
    # unfortunately the Record extension class used for ZSQLMethod
    # results prevents this.

    def _getAttr(self, name):
        return self.__dict__.get(name, None)

    def _setAttr(self, name, value):
        self.__dict__[name] = value

    def getReplies(self):
        """
        Get the list of direct reply ids.
        """
        return self._getAttr('replies')

    def setReplies(self, replies):
        """
        Set the list of direct reply ids.
        """
        self._setAttr('replies', replies)

    def getAttachments(self):
        """
        Get the list of attachment keys.
        """
        return self._getAttr('attachments')

    def setAttachments(self, attachments):
        """
        Set the list of attachment keys.
        """
        self._setAttr('attachments', attachments)

    def getKeywords(self):
        """
        Get the list of keywords for this thread.

        Note that this is stored per thread in the database, not per
        posting!  The actual storage in associated with the root
        posting of the thread in the database.
        """
        return self._getAttr('keywords')

    def setKeywords(self, keywords):
        """
        Set the list of keywords for this thread.

        Note that this is stored per thread in the database, not per
        posting!  The actual storage in associated with the root
        posting of the thread in the database.
        """
        self._setAttr('keywords', keywords)

    def getSyndicated_to(self):
        """
        Get the list of syndication clients for this thread.

        Note that this is stored per thread in the database, not per
        posting!  The actual storage in associated with the root
        posting of the thread in the database.
        """
        return self._getAttr('syndicated_to')

    def setSyndicated_to(self, clients):
        """
        Set the list of syndication clients for this thread.

        Note that this is stored per thread in the database, not per
        posting!  The actual storage in associated with the root
        posting of the thread in the database.
        """
        self._setAttr('syndicated_to', clients)

